
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <malloc.h>
#include <ctype.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <time.h>
#include <linux/ps2/dev.h>

#include "gamecore.h"
#include "harness/harness.h" // temporary, to be removed when code is better

#define _DPRINTF 1
#define DPRINTF if (_DPRINTF) printf


char *format_number(int number); // implemented in main.c

CGameCore::CGameCore()
{
	buffer_length=1024;
	bgpkt=(__u64*)memalign(16, 8*buffer_length);
	angle=0;
	centerx=0;
	centery=0;
}

CGameCore::~CGameCore()
{
	free(bgpkt);
}

int CGameCore::MoveClock(int dx, int dy)
{
	centerx+=dx;
	centery+=dy;

	return 1;
}


int CGameCore::DisplayClock()
{
	static float aa=-M_PI/2.0f; // base angle

	width=(int)(100.0f*cosf(angle+=M_PI/200.0f));
	height=(int)(100.0f*cosf(angle*2.0f));

	int bglen=0;

	*(bgpkt+bglen++)= 0; // top half of GIF tag to be filled at the end to get the size

	*(bgpkt+bglen++)= PS2_GIFTAG_REGS_AD; // bottom half of the GIF tag

	//aa-=M_PI/1000;

	// Add the ticks around the clock, with a long one for every 5 minutes
	{

		int i; // loop counter
		float a; // angle
		float px1, py1, px2, py2; // vertex positions
		float lr=440.0f, br=430.0f, bbr=400.0f; // radius

		i=0;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_RGBAQ( 0x22,0x11,0xFF,0x0,0x0 ); // red, kinda
		*(bgpkt+bglen++)= PS2_GS_RGBAQ;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_PRIM( 	PS2_GS_PRIM_PRIM_LINE,
					PS2_GS_PRIM_IIP_FLAT ,
					PS2_GS_PRIM_TME_OFF,
					PS2_GS_PRIM_FGE_OFF,
					PS2_GS_PRIM_ABE_OFF,
					PS2_GS_PRIM_AA1_OFF,
					PS2_GS_PRIM_FST_UV,
					PS2_GS_PRIM_CTXT_CONTEXT1,
					PS2_GS_PRIM_FIX_NOFIXDDA );
		*(bgpkt+bglen++)= PS2_GS_PRIM;

		a=((float)i)*2*M_PI/60.0f+aa;

		px1=lr*cosf(a);
		py1=lr/har_hdiv*sinf(a);
		if (i%5)
		{
			px2=br*cosf(a);
			py2=br/har_hdiv*sinf(a);
		}
		else
		{
			px2=bbr*cosf(a);
			py2=bbr/har_hdiv*sinf(a);
		}

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,1 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px2)*8)+centerx*16,32768+(((int)py2)*8)+centery*16,1 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_RGBAQ( 0xFF,0xFF,0xFF,0x0,0x0 ); // white
		*(bgpkt+bglen++)= PS2_GS_RGBAQ;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_PRIM( 	PS2_GS_PRIM_PRIM_LINE,
					PS2_GS_PRIM_IIP_FLAT ,
					PS2_GS_PRIM_TME_OFF,
					PS2_GS_PRIM_FGE_OFF,
					PS2_GS_PRIM_ABE_OFF,
					PS2_GS_PRIM_AA1_OFF,
					PS2_GS_PRIM_FST_UV,
					PS2_GS_PRIM_CTXT_CONTEXT1,
					PS2_GS_PRIM_FIX_NOFIXDDA );
		*(bgpkt+bglen++)= PS2_GS_PRIM;

		for (i=1; i<60; i++)
		{
			a=((float)i)*2*M_PI/60.0f+aa;

			px1=lr*cosf(a);
			py1=lr/har_hdiv*sinf(a);
			if (i%5)
			{
				px2=br*cosf(a);
				py2=br/har_hdiv*sinf(a);
			}
			else
			{
				px2=bbr*cosf(a);
				py2=bbr/har_hdiv*sinf(a);
			}

			*(bgpkt+bglen++)=
				PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,1 );
			*(bgpkt+bglen++)= PS2_GS_XYZ2;

			*(bgpkt+bglen++)=
				PS2_GS_SETREG_XYZ2( 32768+(((int)px2)*8)+centerx*16,32768+(((int)py2)*8)+centery*16,1 );
			*(bgpkt+bglen++)= PS2_GS_XYZ2;
		}

	}

	// Add the hands
	{
		struct tm *ttm;
		time_t tt;

		float a; // angle
		float px1, py1, px2, py2; // vertex positions
		float lr=400.0f, llr=200.0f, br=-40.0f; // radius
		float lr2=20.0f, llr2=20.0f, br2=-40.0f; // radius

		time( &tt );
		ttm=localtime(&tt);

		// Add seconds

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_RGBAQ( 0xFF,0xFF,0xFF,0x0,0x0 ); // white
		*(bgpkt+bglen++)= PS2_GS_RGBAQ;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_PRIM( 	PS2_GS_PRIM_PRIM_TRIFAN,
					PS2_GS_PRIM_IIP_FLAT ,
					PS2_GS_PRIM_TME_OFF,
					PS2_GS_PRIM_FGE_OFF,
					PS2_GS_PRIM_ABE_OFF,
					PS2_GS_PRIM_AA1_OFF,
					PS2_GS_PRIM_FST_UV,
					PS2_GS_PRIM_CTXT_CONTEXT1,
					PS2_GS_PRIM_FIX_NOFIXDDA );
		*(bgpkt+bglen++)= PS2_GS_PRIM;

		a=((float)ttm->tm_sec)*2*M_PI/60.0f+aa;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)0)*8)+centerx*16,32768+(((int)0)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		px1=lr*cosf(a);
		py1=lr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*cosf(a);
		py1=lr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*2.0f*cosf(a);
		py1=lr2*2.0f/har_hdiv*sinf(a);
		px2=br2*2.0f*cosf(a);
		py2=br2*2.0f/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*cosf(a);
		py1=lr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr*cosf(a);
		py1=lr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,5 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		// Add minutes

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_RGBAQ( 0x99,0x66,0x99,0x0,0x0 );
		*(bgpkt+bglen++)= PS2_GS_RGBAQ;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_PRIM( 	PS2_GS_PRIM_PRIM_TRIFAN,
					PS2_GS_PRIM_IIP_FLAT ,
					PS2_GS_PRIM_TME_OFF,
					PS2_GS_PRIM_FGE_OFF,
					PS2_GS_PRIM_ABE_OFF,
					PS2_GS_PRIM_AA1_OFF,
					PS2_GS_PRIM_FST_UV,
					PS2_GS_PRIM_CTXT_CONTEXT1,
					PS2_GS_PRIM_FIX_NOFIXDDA );
		*(bgpkt+bglen++)= PS2_GS_PRIM;

		a=((float)ttm->tm_min)*2*M_PI/60.0f+aa;
		a+=((float)ttm->tm_sec)*2*M_PI/3600.0f;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)0)*8)+centerx*16,32768+(((int)0)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		px1=lr*cosf(a);
		py1=lr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*cosf(a);
		py1=lr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*2.0f*cosf(a);
		py1=lr2*2.0f/har_hdiv*sinf(a);
		px2=br2*2.0f*cosf(a);
		py2=br2*2.0f/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr2*cosf(a);
		py1=lr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=lr*cosf(a);
		py1=lr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,3 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		// Add hours

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_RGBAQ( 0x66,0x55,0x99,0x0,0x0 );
		*(bgpkt+bglen++)= PS2_GS_RGBAQ;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_PRIM( 	PS2_GS_PRIM_PRIM_TRIFAN,
					PS2_GS_PRIM_IIP_FLAT ,
					PS2_GS_PRIM_TME_OFF,
					PS2_GS_PRIM_FGE_OFF,
					PS2_GS_PRIM_ABE_OFF,
					PS2_GS_PRIM_AA1_OFF,
					PS2_GS_PRIM_FST_UV,
					PS2_GS_PRIM_CTXT_CONTEXT1,
					PS2_GS_PRIM_FIX_NOFIXDDA );
		*(bgpkt+bglen++)= PS2_GS_PRIM;

		a=((float)ttm->tm_hour)*2.0f*M_PI/12.0f+aa;
		a+=((float)ttm->tm_min)*2.0f*M_PI/12.0f/60.0f;
		a+=((float)ttm->tm_sec)*2.0f*M_PI/12.0f/3600.0f;

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)0)*8)+centerx*16,32768+(((int)0)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		px1=llr*cosf(a);
		py1=llr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=llr2*cosf(a);
		py1=llr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=llr2*2.0f*cosf(a);
		py1=llr2*2.0f/har_hdiv*sinf(a);
		px2=br2*2.0f*cosf(a);
		py2=br2*2.0f/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=llr2*cosf(a);
		py1=llr2/har_hdiv*sinf(a);
		px2=br2*cosf(a);
		py2=br2/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

		a+=M_PI/2.0f;

		px1=llr*cosf(a);
		py1=llr/har_hdiv*sinf(a);
		px2=br*cosf(a);
		py2=br/har_hdiv*sinf(a);

		*(bgpkt+bglen++)=
			PS2_GS_SETREG_XYZ2( 32768+(((int)px1)*8)+centerx*16,32768+(((int)py1)*8)+centery*16,4 );
		*(bgpkt+bglen++)= PS2_GS_XYZ2;

	}

	// write the top half of the GIF tag with calculated size
	*(bgpkt)=
		PS2_GIFTAG_SET_TOPHALF( (bglen*8/16) -1,
				1,1,
				PS2_GS_SETREG_PRIM(	PS2_GS_PRIM_PRIM_SPRITE,
							PS2_GS_PRIM_IIP_FLAT ,
							PS2_GS_PRIM_TME_ON,
							PS2_GS_PRIM_FGE_OFF,
							PS2_GS_PRIM_ABE_OFF,
							PS2_GS_PRIM_AA1_OFF,
							PS2_GS_PRIM_FST_UV,
							PS2_GS_PRIM_CTXT_CONTEXT1,
							PS2_GS_PRIM_FIX_NOFIXDDA ),
				PS2_GIFTAG_FLG_PACKED,1 );


	// ooh, so not the right way to do it...
	assert(bglen<buffer_length);
	if (-1==write( gs_fd, bgpkt, bglen*8 ))
	{
		DPRINTF("write in draw sprite: %s\n", strerror(errno));
		return 0;
	}
	fsync(gs_fd);

	return 1;
}

